package com.hzu.bookingsystem.controller;


import com.hzu.bookingsystem.VO.ResultVO;
import com.hzu.bookingsystem.VO.WeekVO;
import com.hzu.bookingsystem.bean.LabRecordBean;
import com.hzu.bookingsystem.bean.LabTimeBean;
import com.hzu.bookingsystem.converter.Map2Object;
import com.hzu.bookingsystem.dto.LabRecordDTO;
import com.hzu.bookingsystem.repository.LabRecordDTOMapper;
import com.hzu.bookingsystem.service.LabRecordService;
import com.hzu.bookingsystem.service.LabTimeService;
import com.hzu.bookingsystem.utils.ResultVOUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;


@RestController
@CrossOrigin
@RequestMapping("/labrecord")
public class LabRecordController {
    @Autowired
    private LabRecordService labrecordService;

    @Autowired
    private LabTimeService labTimeService;

    @Autowired
    private UserController userController;

//    //    根据id查看实验室记录
//    @GetMapping(value = "/findlabrecord/{lr_id}")
//    public ResultVO<Map<String, LabRecordBean>> FindLabRecordId(@PathVariable("lr_id") Integer lr_id) {
//        LabRecordBean labrecord = labrecordService.findById(lr_id);
//        return ResultVOUtil.success(labrecord);
//    }

//    // 查找所有记录列表
//    @GetMapping(value = "/getlabrecordlist")
//    public ResultVO getLabRecordList() {
//
//        List<LabRecordBean> labrecordList = labrecordService.findAll();
//        return ResultVOUtil.success(labrecordList);
//    }

//    // 查找所有记录审核/未审核
//    @GetMapping(value = "/findstatus/{status}")
//    public ResultVO findStatus(@PathVariable("status") Integer status) {
//
//        List<LabRecordBean> labrecordstatusList = labrecordService.findByStatus(status);
//        return ResultVOUtil.success(labrecordstatusList);
//    }

    // 增加申请记录
    @PostMapping(value = "/addlabrecord", consumes = "application/json")
    public ResultVO addLabRecord(@RequestBody Map<String, Object> map) {
        LabRecordBean labrecord = (LabRecordBean) Map2Object.map2Object(map, LabRecordBean.class);
        labrecordService.add(labrecord);
        return ResultVOUtil.success();
    }

    //更新记录
    @PostMapping(value = "/updatelabrecord", consumes = "application/json")
    public ResultVO updateLabRecord(@RequestBody Map<String, Object> map, HttpServletRequest request) {
        LabRecordBean labrecord = (LabRecordBean) Map2Object.map2Object(map, LabRecordBean.class);
        LabRecordBean labrecordnew = labrecordService.update(labrecord);
        return ResultVOUtil.success(labrecordnew);
    }

//    // 通过lr_id删除记录
//    @PostMapping(value = "/deletelabrecord/{lr_id}", consumes = "application/json")
//    public ResultVO deleteLabRecord(@PathVariable("lr_id") Integer lr_id) {
//        // 查找该用户看是否存在
//        LabRecordBean labrecord = labrecordService.findById(lr_id);
//        if (labrecord == null) {
//            return ResultVOUtil.error(-1, "记录不存在");
//        }
//        // 删除记录
//        labrecordService.deleteById(lr_id);
//        return ResultVOUtil.success();
//    }

    /**
     * author 吴志岳
     * 通过uid查询预约记录
     */
    @GetMapping("/getRecordBySelf")
    public ResultVO getRecordBySelf(HttpServletRequest request) {
        List<LabRecordDTO> labRecordDTOList = labrecordService.findRecordListByUId(userController.getUIdByCookie(request));
        return ResultVOUtil.success(labRecordDTOList);
    }

    /**
     * author 吴志岳
     * 查询预约记录
     */
    @GetMapping("/getRecordList")
    public ResultVO getRecord() {
        List<LabRecordDTO> labRecordDTOList = labrecordService.findRecordList();
        return ResultVOUtil.success(labRecordDTOList);
    }

    /**
     * author 吴志岳
     * 查询预约记录详情
     */
    @GetMapping("/getRecordDetail")
    public ResultVO getRecordDetail(@RequestParam("labId") Integer lab_id,
                                    @RequestParam("year") String year,
                                    @RequestParam("semester") Integer semester,
                                    @RequestParam("day") Integer day,
                                    @RequestParam("time") Integer time,
                                    @RequestParam("uc_id") Integer uc_id) {
        Map<String, Object> map = labrecordService.getRecordDetail(lab_id, uc_id, year, semester, day, time);
        return ResultVOUtil.success(map);
    }

    /**
     * 更改选课时间段
     */
    @PostMapping(value = "/updateWeeks", consumes = "application/json")
    public ResultVO updateWeeks(@RequestBody Map<String, Object> map, HttpServletRequest request) {
        //1. 转换对象
        LabTimeBean labTime = (LabTimeBean) Map2Object.map2Object(map, LabTimeBean.class);
        Integer uc_id = (Integer) map.get("uc_id");
        List<Integer> weeks = (List<Integer>) map.get("weeks");
        System.out.println(labTime);
        System.out.println(uc_id);
        System.out.println(weeks);
        //2. 定义对象
        LabRecordBean labRecordBean = new LabRecordBean();
        labRecordBean.setStatus(0);
        labRecordBean.setUcId(uc_id);

        //3. DAO
        List<Integer> weekAble = labTimeService.findWeeksAble(labTime.getLabId(), labTime.getYear(), labTime.getSemester(), labTime.getDay(), labTime.getTime(), uc_id);
        //判断是否选择不可选周数
        for (Integer i : weeks) {
            if (!weekAble.contains(i))
                return ResultVOUtil.error(-1, "选择了不可选的周数");
        }

        List<LabTimeBean> labTimeBeanList = labTimeService.findTimes(labTime.getLabId(), labTime.getYear(), labTime.getSemester(), labTime.getDay(), labTime.getTime());
        //过滤不可选取
        for (LabTimeBean labTimeBean : labTimeBeanList) {
            if (!weekAble.contains(labTimeBean.getWeek()))
                labTimeBeanList.remove(labTime);
        }

        //4. 删除record
        for (LabTimeBean labTimeBean : labTimeBeanList) {
            labrecordService.deleteByLtIdAndUcId(labTimeBean.getLtId(), uc_id);
        }

        //5. 插入record
        for (LabTimeBean labTimeBean : labTimeBeanList) {
            if (weeks.contains(labTimeBean.getWeek())) {
                LabRecordBean labRecordBean1 = new LabRecordBean().builder()
                        .ltId(labTimeBean.getLtId())
                        .ucId(uc_id)
                        .status(0)
                        .createTime(new Date())
                        .build();
                labrecordService.add(labRecordBean1);
            }
        }
        return ResultVOUtil.success();
    }

    /**
     * 查找用户当前课程已经预约的时间表
     */
    @GetMapping("/getWeekBooked")
    public ResultVO getWeekBooked(@RequestParam("labId") Integer lab_id,
                                  @RequestParam("year") String year,
                                  @RequestParam("semester") Integer semester,
                                  @RequestParam("uc_id") Integer uc_id) {
        //总表
        List<List<List<Integer>>> lists = new ArrayList<>();
        for (int i = 1; i <= 5; i++) {
            // 一行数据
            List<List<Integer>> timeList = new ArrayList<>();
            for (int j = 1; j <= 7; j++) {
                WeekVO weekVO = new WeekVO();

                timeList.add(labrecordService.findWeeksBooked(lab_id, year, semester, j, i, uc_id));
            }
            lists.add(timeList);
        }
        return ResultVOUtil.success(lists);
    }

    /**
     * author 吴志岳
     * 获取实验室课表接口
     *
     * @param lab_id
     * @param year
     * @param semester
     * @return
     */
    @GetMapping("/getLabSchedule")
    public ResultVO getLabSchedule(@RequestParam("labId") Integer lab_id,
                                   @RequestParam("year") String year,
                                   @RequestParam("semester") Integer semester) {
        //总表
        List<List<List<Map<String, Object>>>> lists = new ArrayList<>();
        for (int i = 1; i <= 5; i++) {
            // 一行数据
            List<List<Map<String, Object>>> timeList = new ArrayList<>();
            for (int j = 1; j <= 7; j++) {
                timeList.add(labrecordService.getLabSchedule(lab_id, year, semester, j, i));
            }
            lists.add(timeList);
        }
        return ResultVOUtil.success(lists);
    }

    /**
     * author 吴志岳
     * 获取用户课表接口
     *
     * @param u_id
     * @param year
     * @param semester
     * @return
     */
    @GetMapping("/getUserSchedule")
    public ResultVO getUserSchedule(@RequestParam("u_id") Integer u_id,
                                    @RequestParam("year") String year,
                                    @RequestParam("semester") Integer semester) {
        //总表
        List<List<List<Map<String, Object>>>> lists = new ArrayList<>();
        for (int i = 1; i <= 5; i++) {
            // 一行数据
            List<List<Map<String, Object>>> timeList = new ArrayList<>();
            for (int j = 1; j <= 7; j++) {
                timeList.add(labrecordService.getUserSchedule(u_id, year, semester, j, i));
            }
            lists.add(timeList);
        }
        return ResultVOUtil.success(lists);
    }

    /**
     * author 吴志岳
     * 获取用户课表接口
     *
     * @param year
     * @param semester
     * @return
     */
    @GetMapping("/getUserScheduleBySelf")
    public ResultVO getUserScheduleBySelf(HttpServletRequest request,
                                          @RequestParam("year") String year,
                                          @RequestParam("semester") Integer semester) {
        //总表
        List<List<List<Map<String, Object>>>> lists = new ArrayList<>();
        for (int i = 1; i <= 5; i++) {
            // 一行数据
            List<List<Map<String, Object>>> timeList = new ArrayList<>();
            for (int j = 1; j <= 7; j++) {
                timeList.add(labrecordService.getUserSchedule(userController.getUIdByCookie(request), year, semester, j, i));
            }
            lists.add(timeList);
        }
        return ResultVOUtil.success(lists);
    }

    /**
     * 审核通过
     * author 吴志岳
     */
    @PostMapping(value = "/commit", consumes = "application/json")
    public ResultVO commit(@RequestBody Map<String, Object> map) {
        labrecordService.passRecord((Integer)map.get("lab_id"),
                (Integer)map.get("uc_id"),
                (String) map.get("year"),
                (Integer)map.get("semester"),
                (Integer)map.get("day"),
                (Integer)map.get("time"));
        return ResultVOUtil.success();
    }

    /**
     * 拒绝审核
     * author 吴志岳
     */
    @PostMapping(value = "/refuse", consumes = "application/json")
    public ResultVO refuse(@RequestBody Map<String, Object> map) {
        labrecordService.refuseRecord((Integer)map.get("lab_id"),
                (Integer)map.get("uc_id"),
                (String) map.get("year"),
                (Integer)map.get("semester"),
                (Integer)map.get("day"),
                (Integer)map.get("time"),
                (String)map.get("refuse_reason"));
        return ResultVOUtil.success();
    }

}