package com.hzu.bookingsystem.controller;

import com.hzu.bookingsystem.VO.ResultVO;
import com.hzu.bookingsystem.bean.LabTimeBean;
import com.hzu.bookingsystem.converter.Map2Object;
import com.hzu.bookingsystem.dto.LabTimeDTO;
import com.hzu.bookingsystem.service.LabTimeService;
import com.hzu.bookingsystem.utils.ResultVOUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@RestController
@CrossOrigin
@RequestMapping("/labTime")
public class LabTimeController {
    @Autowired
    private LabTimeService labTimeService;

    @Autowired
    private UserController userController;

    //创建实验室可预约时间段
//    @PostMapping(value = "/addLabTime", consumes = "application/json")
//    public ResultVO addLabTime(@RequestBody Map<String, Object> map) {
//        //转换对象
//        LabTimeBean labTime1 = (LabTimeBean) Map2Object.map2Object(map, LabTimeBean.class);
//        System.out.println(labTime1);
//        //查重
//        if (labTimeService.findByTime(labTime1.getTime()) != null) {
//            return ResultVOUtil.error(-1, "该时间段已存在");
//        }
//        labTimeService.add(labTime1);
//        return ResultVOUtil.success();
//    }

    //通过ltId删除实验室可预约时间段
//    @PostMapping(value = "/deleteLabTime")
//    public ResultVO deleteLabTime(@RequestBody LabTimeBean labTime) {
//        //查找实验室是否存在
//        LabTimeBean labTime1 = labTimeService.findByLtId(labTime.getLtId());
//        if (labTime1 == null) {
//            return ResultVOUtil.error(-1, "该时间段不存在");
//        } else {
//            labTimeService.deleteByLtId(labTime1.getLabId());
//        }
//        return ResultVOUtil.success();
//    }

    //修改实验室可预约时间段
//    @PostMapping(value = "/updateLabTimeOld", consumes = "application/json")
//    public ResultVO updateLabTimeOld(@RequestBody Map<String, Object> map, HttpServletRequest request) {
//        //转换对象
//        LabTimeBean labTime1 = (LabTimeBean) Map2Object.map2Object(map, LabTimeBean.class);
//        if (labTime1.getLtId() == null) {
//            return ResultVOUtil.error(-1, "该实验室不存在，无法修改");
//        } else {
//            labTimeService.update(labTime1);
//        }
//        return ResultVOUtil.success();
//    }

    //通过ltId查找实验室预约时间段
//    @GetMapping(value = "/findLabTimeById")
//    public ResultVO<Map<String, Object>> findLabTimeById(@RequestParam("ltId") Integer lt_id) {
//        LabTimeBean labTime1 = labTimeService.findByLtId(lt_id);
//        System.out.println(labTime1);
//        return ResultVOUtil.success(labTime1);
//    }

    //查找实验室预约周次（包含已预约）
    // author 吴志岳
    @GetMapping(value = "/getWeeks")
    public ResultVO findLabTimeByLabIdAndYearAndSemester(@RequestParam("labId") Integer lab_id,
                                                         @RequestParam("year") String year,
                                                         @RequestParam("semester") Integer semester,
                                                         @RequestParam("day") Integer day,
                                                         @RequestParam("time") Integer time) {
        List<Integer> weeks = labTimeService.findWeeks(lab_id, year, semester, day, time);
        return ResultVOUtil.success(weeks);
    }

    //查找实验室可预约周次（不包含已预约）
    // author 吴志岳
    @GetMapping(value = "/getWeeksAbleBook")
    public ResultVO findLabTimeByLabIdAndYearAndSemesterAbleBook(@RequestParam("labId") Integer lab_id,
                                                                 @RequestParam("year") String year,
                                                                 @RequestParam("semester") Integer semester,
                                                                 @RequestParam("day") Integer day,
                                                                 @RequestParam("time") Integer time,
                                                                 @RequestParam("uc_id") Integer uc_id) {
        List<Integer> weeks = labTimeService.findWeeksAble(lab_id, year, semester, day, time, uc_id);
        return ResultVOUtil.success(weeks);
    }

    //修改实验室可预约时间段
    @PostMapping(value = "/updateLabTime", consumes = "application/json")
    public ResultVO updateLabTime(@RequestBody Map<String, Object> map, HttpServletRequest request) {
        //1. 转换对象，初始化对象，定义时间段
        LabTimeBean labTime = (LabTimeBean) Map2Object.map2Object(map, LabTimeBean.class);
        labTime.setCreatorId(userController.getUIdByCookie(request));
        labTime.setCreateTime(new Date());
        labTime.setStatus(0);
        // 用户发起的修改请求
        List<Integer> weeks = (List<Integer>) map.get("weeks");
        // 实验室预约时间段
        List<Integer> weeksAll = labTimeService.findWeeks(labTime.getLabId(), labTime.getYear(), labTime.getSemester(), labTime.getDay(), labTime.getTime());
        // 实验室可预约时间段
        List<Integer> weeksAble = labTimeService.findWeeksAble(labTime.getLabId(), labTime.getYear(), labTime.getSemester(), labTime.getDay(), labTime.getTime());
        // 实验室不可变换或不可选取的时间段
        List<Integer> weeksDisable = new ArrayList<>(weeksAll);
        weeksDisable.removeAll(weeksAble);

        // 如果请求weeks中不包含不可选取的时间段，报错
        if (!weeks.containsAll(weeksDisable)) {
            return ResultVOUtil.error(-1, "已被预约的时间段无法去除");
        }

        // 2. 数据处理
        // 需要insert的weeks
        List<Integer> weekInsert = new ArrayList<>(weeks);
        weekInsert.removeAll(weeksAll);

        // 需要delete的weeks
        List<Integer> weekDelete = new ArrayList<>(weeksAll);
        weekDelete.removeAll(weeks);

        // 3.批量删除
        List<LabTimeBean> labTimeBeans = labTimeService.findTimes(labTime.getLabId(), labTime.getYear(), labTime.getSemester(), labTime.getDay(), labTime.getTime());
        labTimeBeans.removeIf(labTimeBean -> !weekDelete.contains(labTimeBean.getWeek()));
        for (LabTimeBean labTimeBean : labTimeBeans) {
            labTimeService.deleteByLtId(labTimeBean.getLtId());
        }

        // 4.批量插入
        for (int i : weekInsert) {
            LabTimeBean temp = LabTimeBean.builder().labId(labTime.getLabId())
                    .year(labTime.getYear())
                    .semester(labTime.getSemester())
                    .week(i)
                    .day(labTime.getDay())
                    .time(labTime.getTime())
                    .createTime(labTime.getCreateTime())
                    .creatorId(labTime.getCreatorId())
                    .status(labTime.getStatus())
                    .build();
            labTimeService.add(temp);
        }

        System.out.println("用户请求" + weeks);
        System.out.println("原来有的" + weeksAll);
        System.out.println("要插入的" + weekInsert);
        System.out.println("要删除的" + weekDelete);
        return ResultVOUtil.success(labTimeService.findWeeks(labTime.getLabId(), labTime.getYear(), labTime.getSemester(), labTime.getDay(), labTime.getTime()));
    }


    /**
     * 获取实验室可被预约的【5】【7】时间表
     *
     * @param lab_id
     * @param year
     * @param semester
     * @return
     */
    @GetMapping(value = "/getLabScheduleAll")
    public ResultVO getLabScheduleAll(@RequestParam("labId") Integer lab_id,
                                      @RequestParam("year") String year,
                                      @RequestParam("semester") Integer semester) {
        //总表
        List<List<List<Integer>>> lists = new ArrayList<>();
        for (int i = 1; i <= 5; i++) {
            // 一行数据
            List<List<Integer>> timeList = new ArrayList<>();
            for (int j = 1; j <= 7; j++) {
                timeList.add(labTimeService.findWeeks(lab_id, year, semester, j, i));
            }
            lists.add(timeList);
        }
        return ResultVOUtil.success(lists);
    }

    /**
     * 获取用户可预约的【5】【7】时间表
     *
     * @param lab_id
     * @param year
     * @param semester
     * @return
     */
    @GetMapping(value = "/getLabScheduleAbleBook")
    public ResultVO getLabScheduleAbleBook(@RequestParam("labId") Integer lab_id,
                                           @RequestParam("year") String year,
                                           @RequestParam("semester") Integer semester,
                                           @RequestParam("uc_id") Integer uc_id) {
        //总表
        List<List<List<Integer>>> lists = new ArrayList<>();
        for (int i = 1; i <= 5; i++) {
            // 一行数据
            List<List<Integer>> timeList = new ArrayList<>();
            for (int j = 1; j <= 7; j++) {
                timeList.add(labTimeService.findWeeksAble(lab_id, year, semester, j, i, uc_id));
            }
            lists.add(timeList);
        }
        return ResultVOUtil.success(lists);
    }
//    /**
//     * author 吴志岳
//     * 一键插入该学年学期 16周所有可选时间段
//     */
//    @PostMapping(value = "/fastInsert", consumes = "application/json")
//    public ResultVO fastInsert(@RequestBody Map<String, Object> map, HttpServletRequest request) {
//        //转换对象
//        LabTimeBean labTime1 = (LabTimeBean) Map2Object.map2Object(map, LabTimeBean.class);
//        // 插入16周所有可选时间段
//        for (int i = 1; i <= 7; i++) {
//            for (int j = 1; j <= 5; j++) {
//                for (int k = 1; k <= 16; k++){
//                    labTimeService.add(LabTimeBean.builder()
//                            .labId((Integer)map.get("labId"))
//                            .year(labTime1.getYear()).semester(labTime1.getSemester()).week(k).day(i).time(j)
//                            .creatorId(userController.getUIdByCookie(request))
//                            .status(0)
//                            .createTime(new Date())
//                            .build());
//                }
//            }
//        }
//        return ResultVOUtil.success();
//    }
}
