package com.hzu.bookingsystem.filter;

import com.hzu.bookingsystem.controller.UserController;
import com.hzu.bookingsystem.exception.UserAuthorizeException;
import com.hzu.bookingsystem.exception.UserRuleAuthorizeException;
import com.hzu.bookingsystem.repository.UserSecurityMapper;
import com.hzu.bookingsystem.utils.CookieUtil;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * author 吴志岳
 */
@Aspect
@Component
public class UserAuthorizeAspect {
    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private UserController userController;

    @Resource
    private UserSecurityMapper userSecurityMapper;

    @Pointcut("execution(public * com.hzu.bookingsystem.controller.*.*(..))" +
            "&& !execution(public * com.hzu.bookingsystem.controller.UserController.login(..))" +
            "&& !execution(public * com.hzu.bookingsystem.controller.UserController.logout(..))" +
            "&& !execution(public * com.hzu.bookingsystem.controller.UserController.getUIdByCookie(..))")
//    @Pointcut(
//            "execution(public * com.hzu.bookingsystem.controller.UserController.getUserList(..))")
    public void verify() {
    }

    @Before("verify()")
    public void doVerify() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //查询cookie
        Cookie cookie = CookieUtil.get(request, "token");
        if (cookie == null) {
            System.out.println("【登录校验】Cookie中查不到token");
            throw new UserAuthorizeException("未登录");
        }

        //去redis里查询
        String tokenValue = redisTemplate.opsForValue().get(cookie.getValue());
        if (StringUtils.isEmpty(tokenValue)) {
            System.out.println("登陆已过期");
            throw new UserAuthorizeException("未登录");
        }

        //查询权限
        Map<String, Object> map = userSecurityMapper.getRules(userController.getUIdByCookie(request));
        String rule = (String) map.get("rules_id");
        String[] rules = rule.split(",");
        List<String> ruleList = Arrays.asList(rules);

        String url = request.getRequestURI();
        String[] split = url.split("/");
        String realUrl = "/" + split[1] + "/" + split[2] + "/" + split[3];
        Map<String, Object> map2 = userSecurityMapper.getRuleId(realUrl);

        if (ruleList.contains(((Integer) map2.get("rules_id")).toString())) {
            System.out.println("权限通过");
        } else {
            throw new UserRuleAuthorizeException("无权限访问！");
        }

    }
}
