package com.hzu.bookingsystem.repository;

import com.hzu.bookingsystem.VO.WeekVO;
import com.hzu.bookingsystem.bean.LabRecordBean;
import com.hzu.bookingsystem.bean.LabTimeBean;
import com.hzu.bookingsystem.dto.LabRecordDTO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;

import java.util.List;
import java.util.Map;

@Mapper
public interface LabRecordDTOMapper {
    @Select("SELECT " +
            " tb_lab_time.`year`,  " +
            " tb_lab_time.semester,  " +
            "COUNT(tb_lab_time.`week`) AS `week`," +
            " tb_lab_time.`day`,  " +
            " tb_lab_time.time,  " +
            " tb_lab_record.`status`,  " +
            " tb_user_course.`name` AS course_name,  " +
            " tb_user_course.population,  " +
            " tb_user_course.classes,  " +
            " tb_lab.`name` AS lab_name,  " +
            " tb_lab.position " +
            "FROM " +
            " tb_lab_time " +
            " INNER JOIN " +
            " tb_lab_record " +
            " ON  " +
            "  tb_lab_time.lt_id = tb_lab_record.lt_id " +
            " INNER JOIN " +
            " tb_lab " +
            " ON  " +
            "  tb_lab_time.lab_id = tb_lab.lab_id, " +
            " tb_user_course " +
            "WHERE " +
            " tb_lab_time.lt_id = tb_lab_record.lt_id AND " +
            " tb_lab_record.uc_id = tb_user_course.uc_id AND " +
            " tb_lab_time.lab_id = tb_lab.lab_id AND " +
            " tb_user_course.u_id =  #{uId} " +
            "GROUP BY " +
            " tb_lab_record.uc_id, " +
            " tb_lab_time.`year`,  " +
            " tb_lab_time.semester,  " +
            " tb_lab_time.`day`,  " +
            " tb_lab_time.time,  " +
            " tb_lab_record.`status`" +
            "ORDER BY tb_lab_record.create_time desc")
    List<LabRecordDTO> getRecordListByUId(Integer uId);

    @Select("SELECT " +
            " tb_lab_time.`year`, " +
            " tb_lab_time.semester, " +
            " COUNT( tb_lab_time.`week` ) AS `week`, " +
            " tb_lab_time.`day`, " +
            " tb_lab_time.time, " +
            " tb_lab_record.`status`, " +
            " tb_user_course.`name` AS course_name, " +
            " tb_user_course.population, " +
            " tb_user_course.classes, " +
            " tb_user_course.uc_id, " +
            " tb_lab.`name` AS lab_name, " +
            " tb_lab.lab_id AS labId, " +
            " tb_lab.position  " +
            "FROM " +
            " tb_lab_time " +
            " INNER JOIN tb_lab_record ON tb_lab_time.lt_id = tb_lab_record.lt_id " +
            " INNER JOIN tb_lab ON tb_lab_time.lab_id = tb_lab.lab_id, " +
            " tb_user_course  " +
            "WHERE " +
            " tb_lab_time.lt_id = tb_lab_record.lt_id  " +
            " AND tb_lab_record.uc_id = tb_user_course.uc_id  " +
            " AND tb_lab_time.lab_id = tb_lab.lab_id  " +
            "GROUP BY " +
            " tb_lab_record.uc_id, " +
            " tb_lab_time.`year`, " +
            " tb_lab_time.semester, " +
            " tb_lab_time.`day`, " +
            " tb_lab_time.time, " +
            " tb_lab_record.`status`  " +
            "ORDER BY " +
            " tb_lab_record.create_time DESC")
    List<LabRecordDTO> getRecordList();


    @Select("SELECT " +
            " tb_lab_time.`week`  " +
            "FROM " +
            " tb_user_course, " +
            " tb_lab_record " +
            " INNER JOIN tb_lab_time ON tb_lab_record.lt_id = tb_lab_time.lt_id  " +
            "WHERE " +
            " tb_lab_time.`year` = #{year}  " +
            " AND tb_lab_time.semester = #{semester} " +
            " AND tb_lab_time.lab_id = #{labId} " +
            " AND tb_lab_time.`day` = #{day} " +
            " AND tb_lab_time.time = #{time}  " +
            " AND tb_user_course.uc_id = tb_lab_record.uc_id  " +
            " AND tb_user_course.uc_id = #{uc_id} " +
            " AND tb_lab_record.`status` IN ( " +
            " 0, " + " 2)")
    List<WeekVO> getWeekBooked(Integer labId, String year, Integer semester, Integer day, Integer time, Integer uc_id);


    @Select("SELECT " +
            " tb_user_course.`name` AS course_name, " +
            " tb_user.nickname, " +
            " tb_user_course.classes, " +
            " tb_user_course.population, " +
            " tb_user_course.software, " +
            " tb_user_course.uc_id  " +
            "FROM " +
            " tb_lab " +
            " INNER JOIN tb_lab_time ON tb_lab.lab_id = tb_lab_time.lab_id " +
            " INNER JOIN tb_lab_record ON tb_lab_record.lt_id = tb_lab_time.lt_id " +
            " INNER JOIN tb_user " +
            " INNER JOIN tb_user_course ON tb_user.u_id = tb_user_course.u_id  " +
            "WHERE " +
            " tb_lab_time.lt_id = tb_lab_record.lt_id  " +
            " AND tb_lab_record.uc_id = tb_user_course.uc_id  " +
            " AND tb_lab.lab_id = #{labId}" +
            " AND tb_lab_time.`year` = #{year}  " +
            " AND tb_lab_time.semester = #{semester}  " +
            " AND tb_lab_time.`day` = #{day}  " +
            " AND tb_lab_time.time = #{time} " +
            " AND tb_lab_record.status = 2  " +
            "GROUP BY " +
            " tb_user_course.uc_id")
    List<Map<String, Object>> getLabSchedule(Integer labId, String year, Integer semester, Integer day, Integer time);

    @Select("SELECT " +
            " tb_user_course.`name` AS course_name, " +
            " tb_user.nickname, " +
            " tb_user_course.classes, " +
            " tb_user_course.population, " +
            " tb_user_course.software, " +
            " tb_user_course.uc_id, " +
            " tb_lab.lab_id " +
            "FROM " +
            " tb_lab " +
            " INNER JOIN tb_lab_time ON tb_lab.lab_id = tb_lab_time.lab_id " +
            " INNER JOIN tb_lab_record ON tb_lab_record.lt_id = tb_lab_time.lt_id " +
            " INNER JOIN tb_user " +
            " INNER JOIN tb_user_course ON tb_user.u_id = tb_user_course.u_id  " +
            "WHERE " +
            " tb_lab_time.lt_id = tb_lab_record.lt_id  " +
            " AND tb_lab_record.uc_id = tb_user_course.uc_id  " +
            " AND tb_lab_time.`year` = #{year}  " +
            " AND tb_lab_time.semester = #{semester}  " +
            " AND tb_lab_time.`day` = #{day}  " +
            " AND tb_lab_time.time = #{time} " +
            " AND tb_user.u_id = #{u_id}  " +
            " AND tb_lab_record.status = 2  " +
            "GROUP BY " +
            " tb_user_course.uc_id")
    List<Map<String, Object>> getUserSchedule(Integer u_id, String year, Integer semester, Integer day, Integer time);

    /**
     * 通过lab_id,uc_id以及4项时间查找审核详情
     */
    @Select("SELECT " +
            " tb_lab.lab_id, " +
            " tb_lab_time.year, " +
            " tb_lab_time.semester*1 AS semester, " +
            " tb_lab_time.day*1 AS day, " +
            " tb_lab_time.time*1 AS time, " +
            " tb_user_course.uc_id, "+
            " tb_lab_record.create_time, " +
            " tb_user.nickname, " +
            " tb_user.tel, " +
            " tb_lab.position, " +
            " tb_user_course.software, " +
            " tb_user_course.population, " +
            " tb_user_course.classes, " +
            " tb_user_course.remind, " +
            " tb_user_course.isHaveDocuments, " +
            " tb_lab.`name` AS lab_name, " +
            " tb_user_course.`name` AS course_name  " +
            "FROM " +
            " tb_lab " +
            " INNER JOIN tb_lab_time ON tb_lab.lab_id = tb_lab_time.lab_id " +
            " INNER JOIN tb_lab_record ON tb_lab_time.lt_id = tb_lab_record.lt_id " +
            " INNER JOIN tb_user " +
            " INNER JOIN tb_user_course ON tb_user.u_id = tb_user_course.u_id  " +
            "WHERE " +
            " tb_lab_record.uc_id = tb_user_course.uc_id  " +
            " AND tb_lab_time.`year` = #{year}  " +
            " AND tb_lab_time.semester = #{semester} " +
            " AND tb_lab_time.`day` = #{day}  " +
            " AND tb_lab_time.time = #{time}  " +
            " AND tb_lab.lab_id = #{labId}  " +
            " AND tb_user_course.uc_id = #{uc_id}  " +
            "GROUP BY " +
            " tb_lab.lab_id, " +
            " tb_lab_time.`year`, " +
            " tb_lab_time.semester, " +
            " tb_lab_time.`day`, " +
            " tb_lab_time.time")
    Map<String, Object> getRecordDetail(Integer labId, Integer uc_id, String year, Integer semester, Integer day, Integer time);

    /**
     * author 吴志岳
     * 查找审核详情所有符合的lr_id
     */
    @Select("SELECT " +
            " tb_lab_record.lr_id as lrId, tb_lab_record.lt_id as ltId, tb_lab_record.uc_id as ucId, tb_lab_record.status,tb_lab_record.refuse_reason as refuseReason,tb_lab_record.check_time as checkTime,tb_lab_record.create_time as createTime " +
            "FROM " +
            " tb_lab " +
            " INNER JOIN tb_lab_time ON tb_lab.lab_id = tb_lab_time.lab_id " +
            " INNER JOIN tb_lab_record ON tb_lab_time.lt_id = tb_lab_record.lt_id " +
            " INNER JOIN tb_user " +
            " INNER JOIN tb_user_course ON tb_user.u_id = tb_user_course.u_id  " +
            "WHERE " +
            " tb_lab_record.uc_id = tb_user_course.uc_id  " +
            " AND tb_lab_record.`status` = 0  " +
            " AND tb_lab_time.`year` = #{year}  " +
            " AND tb_lab_time.semester = #{semester} " +
            " AND tb_lab_time.`day` = #{day}  " +
            " AND tb_lab_time.time = #{time}  " +
            " AND tb_lab.lab_id = #{labId}  " +
            " AND tb_user_course.uc_id = #{uc_id}  ")
    List<LabRecordBean> getRecordDetailLabRecordList(Integer labId, Integer uc_id, String year, Integer semester, Integer day, Integer time);

    @Select("SELECT " +
            " tb_lab_time.*,tb_lab_time.lt_id as ltId,tb_lab_time.lab_id as labId,tb_lab_time.creator_id as creatorId "+
            " FROM " +
            " tb_lab " +
            " INNER JOIN tb_lab_time ON tb_lab.lab_id = tb_lab_time.lab_id " +
            " INNER JOIN tb_lab_record ON tb_lab_time.lt_id = tb_lab_record.lt_id " +
            " INNER JOIN tb_user " +
            " INNER JOIN tb_user_course ON tb_user.u_id = tb_user_course.u_id  " +
            "WHERE " +
            " tb_lab_record.uc_id = tb_user_course.uc_id  " +
            " AND tb_lab_record.`status` = 0  " +
            " AND tb_lab_time.`year` = #{year}  " +
            " AND tb_lab_time.semester = #{semester} " +
            " AND tb_lab_time.`day` = #{day}  " +
            " AND tb_lab_time.time = #{time}  " +
            " AND tb_lab.lab_id = #{labId}  " +
            " AND tb_user_course.uc_id = #{uc_id}  ")
    List<LabTimeBean> getRecordDetailLabTimeList(Integer labId, Integer uc_id, String year, Integer semester, Integer day, Integer time);
}
