package com.hzu.bookingsystem.repository;


import com.hzu.bookingsystem.bean.LabTimeBean;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;

import java.util.List;

/**
 * author 莫家豪
 */
public interface LabTimeRepository extends JpaRepository<LabTimeBean,String> {

    /**
     * 通过lt_id删除实验室
     *
     * @param ltId
     * @return
     */
    void deleteByLtId(Integer ltId);

    // 查找所有实验室可预约时间段
    List<LabTimeBean> findAll();

    /**
     * 通过lt_id查找实验室可预约时间段
     *
     * @param ltId
     * @return
     */
    LabTimeBean findByLtId(Integer ltId);

    /**
     * 通过状态查找实验室可预约时间段
     *
     * @param status
     * @return
     */
    LabTimeBean findByStatus(Integer status);

    /**
     * 通过上课时间查找实验室可预约时间段
     *
     * @param time
     * @return
     */
    LabTimeBean findByTime(Integer time);

    /**
     * 通过学期、周、星期、上课时间查找实验室预约时间段
     *
     * @param lab_id、year、semester
     * @return
     */
    @Query(value = "SELECT " +
            " tb_lab_time.* " +
            "FROM " +
            "tb_lab_time LEFT JOIN tb_lab_record ON tb_lab_time.lt_id=tb_lab_record.lt_id " +
            "WHERE " +
            " tb_lab_time.lab_id = ?1 AND " +
            " tb_lab_time.`year` = ?2 AND " +
            " tb_lab_time.semester = ?3 AND " +
            " tb_lab_time.`day` = ?4 AND " +
            " tb_lab_time.time = ?5 AND " +
            "tb_lab_time.lt_id not in (select lt_id from tb_lab_record where status = 1)", nativeQuery = true)
    List<LabTimeBean> findByLabIdAndYearAndSemesterAndDayAndTime(Integer lab_id,String year,Integer semester, Integer day,Integer time) ;

    /**
     * 通过学期、周、星期、上课时间查找实验室可预约时间段
     *
     * @param lab_id、year、semester
     * @return
     */
    @Query(value = "SELECT " +
            " tb_lab_time.* " +
            "FROM " +
            "tb_lab_time LEFT JOIN tb_lab_record ON tb_lab_time.lt_id=tb_lab_record.lt_id " +
            "WHERE " +
            " tb_lab_time.lab_id = ?1 AND " +
            " tb_lab_time.`year` = ?2 AND " +
            " tb_lab_time.semester = ?3 AND " +
            " tb_lab_time.`day` = ?4 AND " +
            " tb_lab_time.time = ?5 AND " +
            " tb_lab_time.lt_id not in (select lt_id from tb_lab_record where status is not null)" , nativeQuery = true)
    List<LabTimeBean> findWeeksAbleBook(Integer lab_id, String year, Integer semester, Integer day, Integer time) ;

    @Query(value = "SELECT " +
            " tb_lab_time.* " +
            "FROM " +
            "tb_lab_time LEFT JOIN tb_lab_record ON tb_lab_time.lt_id=tb_lab_record.lt_id " +
            "WHERE " +
            " tb_lab_time.lab_id = ?1 AND " +
            " tb_lab_time.`year` = ?2 AND " +
            " tb_lab_time.semester = ?3 AND " +
            " tb_lab_time.`day` = ?4 AND " +
            " tb_lab_time.time = ?5 AND " +
            " tb_lab_time.lt_id not in (select lt_id from tb_lab_record where status is not null)" +
            "OR " +
            " tb_lab_time.lab_id = ?1 AND " +
            " tb_lab_time.`year` = ?2 AND " +
            " tb_lab_time.semester = ?3 AND " +
            " tb_lab_time.`day` = ?4 AND " +
            " tb_lab_time.time = ?5 AND " +
            "tb_lab_record.uc_id = ?6" , nativeQuery = true)
    List<LabTimeBean> findWeeksAbleBook(Integer lab_id, String year, Integer semester, Integer day, Integer time,Integer uc_id) ;

}
