package com.hzu.bookingsystem.service;


import com.hzu.bookingsystem.bean.LabRecordBean;
import com.hzu.bookingsystem.dto.LabRecordDTO;

import java.util.List;
import java.util.Map;

public interface LabRecordService {


    /**
     * 新增申请记录
     *
     * @param labrecord
     * @return
     */
    LabRecordBean add(LabRecordBean labrecord);

    /**
     * 查看所有申请记录
     *
     * @return
     */
    List<LabRecordBean> findAll();

    /**
     * 根据状态查询申请记录
     *
     * @param stauts
     * @return
     */
    List<LabRecordBean> findByStatus(Integer stauts);

    /**
     * 根据ID查询记录
     *
     * @param lr_id
     * @return
     */
    LabRecordBean findById(Integer lr_id);

    /**
     * 修改申请记录
     *
     * @param labrecord
     * @return
     */
    LabRecordBean update(LabRecordBean labrecord);

    /**
     * 根据Id查询申请记录
     *
     * @param lr_id
     */
    void deleteById(Integer lr_id);

    /**
     * 根据ucid查出实验室记录
     *
     * @param uc_id
     * @return
     */
    List<LabRecordBean> findByUcIdAndStatus(Integer uc_id, Integer status);

    /**
     * 根据教师id和学年，学期查询课表
     *
     * @param u_id
     * @param year
     * @param semester
     * @return
     */
    String[][] getcourseByuser(Integer u_id, String year, Integer semester);


    /**
     * author 吴志岳
     * 查看自己的审核记录
     */
    List<LabRecordDTO> findRecordListByUId(Integer uId);

    /**
     * author 吴志岳
     * 查看审核记录
     */
    List<LabRecordDTO> findRecordList();

    /**
     * author 吴志岳
     * 根据LTid与UCid删除LR
     */
    void deleteByLtIdAndUcId(Integer LtId, Integer UcId);

    /**
     * author 吴志岳
     * 查找用户当前课程已经预约的时间段
     */
    List<Integer> findWeeksBooked(Integer labId, String year, Integer semester, Integer day, Integer time, Integer uc_id);

    /**
     * 吴志岳
     * 获取实验室课表信息
     */
    List<Map<String, Object>> getLabSchedule(Integer labId, String year, Integer semester, Integer day, Integer time);

    /**
     * 吴志岳
     * 获取用户课表信息
     */
    List<Map<String, Object>> getUserSchedule(Integer u_id, String year, Integer semester, Integer day, Integer time);

    /**
     * author 吴志岳
     * 获取审核详情
     */
    Map<String, Object> getRecordDetail(Integer labId,Integer uc_id, String year, Integer semester, Integer day, Integer time);


    /**
     * 审核通过
     * author 吴志岳
     */
    void passRecord(Integer labId,Integer uc_id, String year, Integer semester, Integer day, Integer time);

    /**
     * 拒绝审核
     * author 吴志岳
     */
    void refuseRecord(Integer labId,Integer uc_id, String year, Integer semester, Integer day, Integer time,String refuseReason);

}
