package com.hzu.bookingsystem.service;


import com.hzu.bookingsystem.bean.LabTimeBean;
import com.hzu.bookingsystem.dto.LabTimeDTO;

import java.util.List;
/**
 * author 莫家豪
 */
public interface LabTimeService {

    /**
     * 创建实验室可申请时间段
     *
     * @param labTime
     * @return
     */
    LabTimeBean add(LabTimeBean labTime);

    /**
     * 通过lt_id删除实验室
     *
     * @param ltID
     * @return
     */
    void deleteByLtId(Integer ltID);

    /**
     * 修改实验室
     *
     * @param labTime
     * @return
     */
    LabTimeBean update(LabTimeBean labTime);

    // 查找所有用户
    List<LabTimeBean> findAll();

    /**
     * 通过lt_id查找实验室可预约时间段
     *
     * @param ltId
     * @return
     */
    LabTimeBean findByLtId(Integer ltId);

    /**
     * 通过状态查找实验室可预约时间段
     *
     * @param status
     * @return
     */
    LabTimeBean findByStatus(Integer status);

    /**
     * 通过上课时间查找实验室可预约时间段
     *
     * @param time
     * @return
     */
    LabTimeBean findByTime(Integer time);

    /**
     * 查找所有实约时间信息
     * @return
     */
    List<LabTimeDTO> findAllLabInfo();

    /**
     * 通过学期、周、星期、上课时间查找实验室预约时间段周次
     *author 吴志岳
     * @param lab_id、year、semester
     * @return List<Integer>
     */
    List<Integer> findWeeks(Integer lab_id,String year,Integer semester, Integer day,Integer time) ;

    /**
     * 通过学期、周、星期、上课时间查找实验室可预约时间段周次
     *author 吴志岳
     * @param lab_id、year、semester
     * @return List<Integer>
     */
    List<Integer> findWeeksAble(Integer lab_id,String year,Integer semester, Integer day,Integer time) ;

    /**
     * 通过学期、周、星期、上课时间、ucid查找实验室可预约时间段周次
     *author 吴志岳
     * @param lab_id、year、semester
     * @return List<Integer>
     */
    List<Integer> findWeeksAble(Integer lab_id,String year,Integer semester, Integer day,Integer time,Integer uc_id) ;

    /**
     * 通过学年、学期、星期、上课时间查找实验室预约时间段
     */
    List<LabTimeBean> findTimes(Integer lab_id,String year,Integer semester, Integer day,Integer time);
}