package com.hzu.bookingsystem.service.impl;

import com.hzu.bookingsystem.bean.LabTimeBean;
import com.hzu.bookingsystem.dto.LabTimeDTO;
import com.hzu.bookingsystem.repository.LabTimeRepository;
import com.hzu.bookingsystem.service.LabTimeService;
import org.hibernate.SQLQuery;
import org.hibernate.transform.Transformers;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

/**
 * author 莫家豪
 */
@Service
public class LabTimeServiceImpl implements LabTimeService {
    @Resource
    private LabTimeRepository labTimeRepository;

    @PersistenceContext
    private EntityManager entityManager;

    @Override
    public LabTimeBean add(LabTimeBean labTime) {
        labTime.setLtId(null);
        return labTimeRepository.save(labTime);
    }

    @Override
    @Transactional
    public void deleteByLtId(Integer ltId) {
        labTimeRepository.deleteByLtId(ltId);
    }

    @Override
    public LabTimeBean update(LabTimeBean labTime) {
        return labTimeRepository.save(labTime);
    }

    @Override
    public List<LabTimeBean> findAll() {
        return labTimeRepository.findAll();
    }

    @Override
    public LabTimeBean findByLtId(Integer ltId) {
        return labTimeRepository.findByLtId(ltId);
    }

    @Override
    public LabTimeBean findByStatus(Integer status) {
        return labTimeRepository.findByStatus(status);
    }

    @Override
    public LabTimeBean findByTime(Integer time) {
        return labTimeRepository.findByTime(time);
    }


    @Override
    public List<LabTimeDTO> findAllLabInfo() {
        String sql = "SELECT tb_lab_time.lt_id as LtId," +
                "tb_lab_time.year," +
                "tb_lab_time.semester," +
                "tb_lab_time.week," +
                "tb_lab_time.day," +
                "tb_lab_time.time," +
                "tb_lab_time.status," +
                "tb_lab_time.remind " +
                "FROM " +
                "tb_lab_time ";
        Query query = entityManager.createNativeQuery(sql);
        query.unwrap(SQLQuery.class).setResultTransformer(Transformers.aliasToBean(LabTimeDTO.class));
        List<LabTimeDTO> LabTimeDTOList = query.getResultList();
        return LabTimeDTOList;
    }

    /**
     * author 吴志岳
     *
     * @param lab_id、year、semester
     * @param year
     * @param semester
     * @return
     */
    @Override
    public List<Integer> findWeeks(Integer lab_id, String year, Integer semester, Integer day, Integer time) {
        List<LabTimeBean> list = labTimeRepository.findByLabIdAndYearAndSemesterAndDayAndTime(lab_id, year, semester, day, time);
        List<Integer> weeks = new ArrayList<>();
        for (LabTimeBean labtime : list) {
            weeks.add(labtime.getWeek());
        }
        return weeks;
    }

    /**
     * author 吴志岳
     *
     * @param lab_id、year、semester
     * @param year
     * @param semester
     * @return
     */
    @Override
    public List<Integer> findWeeksAble(Integer lab_id, String year, Integer semester, Integer day, Integer time) {
        List<LabTimeBean> list = labTimeRepository.findWeeksAbleBook(lab_id, year, semester, day, time);
        List<Integer> weeks = new ArrayList<>();
        for (LabTimeBean labtime : list) {
            weeks.add(labtime.getWeek());
        }
        return weeks;
    }

    /**
     * author 吴志岳
     *
     * @param lab_id、year、semester
     * @param year
     * @param semester
     * @return
     */
    @Override
    public List<Integer> findWeeksAble(Integer lab_id, String year, Integer semester, Integer day, Integer time,Integer uc_id) {
        List<LabTimeBean> list = labTimeRepository.findWeeksAbleBook(lab_id, year, semester, day, time, uc_id);
        List<Integer> weeks = new ArrayList<>();
        for (LabTimeBean labtime : list) {
            weeks.add(labtime.getWeek());
        }
        return weeks;
    }

    @Override
    public List<LabTimeBean> findTimes(Integer lab_id, String year, Integer semester, Integer day, Integer time) {
        return labTimeRepository.findByLabIdAndYearAndSemesterAndDayAndTime(lab_id, year, semester, day, time);
    }

}